

/* Copyright(C) (2002) (ID - IMAG) <Florent.Morata@imag.fr> 

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 


/*! \file psi_sample.cpp
 * \brief PSI Simulation Main.
 * \author - Florent.Morata@imag.fr
		   - Corine.Marchand@imag.fr
 *         - Jean-Marc.Vincent@imag.fr
 * \version November 26, 2002
 */
 

 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 
 // C++ includes
 #include <iostream>
 #include <string>
 #include <vector>
 #include <algorithm>
 #include <fstream>
 
 // C include 
 #include <cstdlib>
 
 /*! \namespace std
  * \brief Utilization of the Standard Template Library.
  */
 using namespace std;
 
 
 // Unix include 
 #include <unistd.h>
 
 // local includes
 #include "alea.h"
 #include "alias.h"
 #include "cost.h"
 #include "hbf.h"
 
 // global variable
 /*! Executable name. */
 char *prog; 
 //--------------------------------------------------------------------------------------------------------------
 
 
 /*! \fn void backward(const string path,const int size,const string file_out,const string path_cost)
  * \brief PSI Exact Simulation. 
  *
  * This is the exact simulation kernel of PSI.
  * \param path a string corresponding to the model name.
  * \param size an integer corresponding to the sample size.
  * \param file_out a string corresponding to the (optional) output sample path. 
  * \param path_cost a string corresponding to the full input (optional) cost path.
  */
void backward(const string path,const int size,const string file_out,const string path_cost)
{
  Alias A;
  Cost C;
  int stop_time;
  ofstream out;
	
  string path_out = file_out + ".sample"; // output path		
  out.open(path_out.c_str());

  int D = A.read(path);      // reading of aliasing data
  int N = A.size() / D;

    
  if(path_cost != path) C.mk_cost(path_cost); 
    else C.init_state(N);
  

  out << "################################################### " << endl;
  out << "#                PSI 2002 (backward simulation)   # " << endl;
  out << "#                                                 # " << endl;
  out << "#              sample's size                      # " << endl;
  out << "#                                                 # " << endl;
  out << "#        Cost + iterations                        # " << endl;
  out << "##################################################! " << endl;
  out << " " << endl;
  
  
  init_generator(); // initialization of random generator
  
  out <<  size << endl;
  out << " " << endl;

  int d = 0; // simulation duration
  
  
  while(d != size)
    {
      C.init();
      stop_time = 0;
      
      do
	{
	  stop_time++;
	  
	  double rand = uniform();   // Random called
	  int j = (int) (D*uniform());
	  
	  for(int i = 0; i < N; i++)         
	    C.change(i,A.walk(rand,j,N,D,i)); // Aliasing generation of (backward) trajectories
	  
	  C.swap(); // cost vectors update
	  
	} while(C.test() == true);
      
      out << C.value() << " " << stop_time << endl;
      C.free_cl();
      d++;
      
    }
  
  out.close(); 
	
  cout << " File : '" << path_out << "' created ...\n";

}

//--------------------------------------------------------------------------------------------------------------


/*! \fn void help_sample()
 * \brief Help message of psi_sample executable.
 */
void help_sample()
{
  cerr << " USAGE : " << prog << " [-idoc] argument [-hv]\n";
  cerr << "\n  -i : input 'simu' file (full path)\n";
  cerr << "\n  -d : sample size\n";
  cerr << "\n  -o : output file (automatically generate with 'sample' extension)\n";
  cerr << "\n       By default, output file has the same name than input file.\n";
  cerr << "\n  -c : input cost file (full path)\n";
  cerr << "\n       If -c option is not specified, cost function is the space states.\n";
  cerr << "\n  -h : help.\n";
  cerr << "\n  -v : version\n"; 
  cerr << " " << endl;
}

//--------------------------------------------------------------------------------------------------------------

/*! \fn void version_sample()
 * \brief Give PSI informations.
 */
void version_sample()
{
  cout << " P.S.I. version 1.0 - Exact sampling\n";
  cout << " Authors : Florent.Morata@imag.fr\n";
  cout << "           Corine.Marchand@imag.fr\n";
  cout << "           Jean-Marc.Vincent@imag.fr\n";
  cout << " Web : http://www-id.imag.fr\n\n";
}

//--------------------------------------------------------------------------------------------------------------


/*! \fn int main(int argc,char **argv)
 * \brief PSI Simulation Main Program.
 *
 * Unix like version of PSI simulation.
 */	
int main(int argc,char **argv)
{ 
  char c;
  string ext,opt_i,opt_o,opt_c;
  char *opt_d;
  prog = argv[0];	
  int compt_o = 0;
  int compt_c = 0;
  int compt_h = 0;
  int compt_i = 0;
	
  cout << " " << endl;
  while((c = getopt(argc,argv,"hvi:o:d:c:")) != -1)
    {
      switch(c)
	 {
	 case 'h': help_sample(); 
	   compt_h ++;
	   compt_i ++; // to avoid repetition help message
	   break;

	 case 'v': version_sample();
	   compt_h ++;
	   compt_i ++; // to avoid repetition help message
	   break;
	 
	 case 'i' : opt_i = optarg;
	   compt_i ++;
	   break;
	
	 case 'o' : opt_o = optarg;
	   compt_o ++; // utilization of -o option
	   break;
	
	 case 'd' : opt_d = optarg;
	   break;
	
	 case 'c' : opt_c = optarg;
	   compt_c ++; // utilization of -c option
	   break;

	 }
    }

  if((c == -1) && (compt_i == 0)) // no option specified
    {
      help_sample();
      compt_h++;
    }

  else
    {
      if(compt_h == 0)  // backward simulation called
	{
	  if(compt_o == 0 && compt_c == 0) backward(opt_i,atoi(opt_d),opt_i.substr(0,opt_i.find('.')),opt_i); // no optional argument
	  if(compt_o == 0 && compt_c == 1) backward(opt_i,atoi(opt_d),opt_c.substr(0,opt_c.find('.')),opt_c); // -c argument
	  if(compt_o == 1 && compt_c == 0) backward(opt_i,atoi(opt_d),opt_o,opt_i); // -o argument
	  if(compt_o == 1 && compt_c == 1) backward(opt_i,atoi(opt_d),opt_o,opt_c); // -o & -c arguments
	}
      
    }
  
}

 //--------------------------------------------------------------------------------------------------------------










